import ConstructorOf from '@thats/utils/types/ConstructorOf'
import { Material, MeshLambertMaterial } from 'three'
import XRMass from '../../core/XRMass'

export interface IFocusable extends XRMass {
	focusMaterial: Material
	isFocused: boolean
	isFocusable: boolean

	focus(): void
	blur(): void
	onChangeFocus(v: boolean): void
}

export default function Focusable<T extends XRMass>(
	Base: ConstructorOf<T>
): ConstructorOf<T & IFocusable> {
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class FocusableMixIn extends Base {
		focusMaterial: MeshLambertMaterial
		isFocused: boolean
		isFocusable: boolean

		defaults() {
			return {
				...super.defaults(),
				isFocusable: true,
				isFocused: false,
				focusMaterial: new MeshLambertMaterial({
					transparent: true,
					opacity: 0.8,
					userData: { importance: 1 },
				}),
			}
		}

		init() {
			super.init()
			this.focusMaterial.color = this.material.color
		}

		focus() {
			if (this.isFocused) return
			this.isFocused = true
			// this.layers.enable(1)
			this.setMaterial(this.focusMaterial)
			this.onChangeFocus(true)
		}

		blur() {
			if (!this.isFocused) return
			this.isFocused = false
			// this.layers.disable(1)
			this.removeMaterial(this.focusMaterial)
			this.onChangeFocus(false)
		}

		onChangeFocus(v: boolean) {}
	}
	return FocusableMixIn as unknown as ConstructorOf<T & IFocusable>
}
