import ConstructorOf from '@thats/utils/types/ConstructorOf'
import { IFocusable } from '../focuser/Focusable'

export interface IGrabbable extends IFocusable {
	isGrabbable: boolean
	isGrabbed: boolean

	grab(): void
	ungrab(): void
	onChangeGrabbed(v: boolean): void
}

export default function Grabbable<T extends IFocusable>(
	Base: ConstructorOf<T>
): ConstructorOf<T & IGrabbable> {
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class GrabbableMixIn extends Base {
		isGrabbable: boolean
		isGrabbed: boolean

		defaults() {
			return {
				...super.defaults(),
				isGrabbable: true,
				isGrabbed: false,
			}
		}

		grab() {
			if (this.isGrabbed) return
			this.isGrabbed = true
			this.turnOnTempKinematics()
			this.onChangeGrabbed(true)
		}
		ungrab() {
			if (!this.isGrabbed) return
			this.isGrabbed = false
			this.turnOffTempKinematics()
			this.onChangeGrabbed(false)
		}

		onChangeGrabbed(v: boolean) {}
	}
	return GrabbableMixIn as unknown as ConstructorOf<T & IGrabbable>
}
