import ConstructorOf from '@thats/utils/types/ConstructorOf'
import { Vector2 } from 'three'
import XRController, { XRControllerConstructor } from '../../core/XRController'
import ControllerType from '../../enums/ControllerType'
import Selector, { ISelector } from '../selector/Selector'
import { IGrabbable } from './Grabbable'
import GrabberKeyboard from './GrabberKeyboard'
import GrabberQuest2 from './GrabberQuest2'

export interface IGrabber extends XRController {
	grabbed: IGrabbable[]

	drag(v: Vector2): void
	grab(v: IGrabbable): void
	ungrab(v: IGrabbable): void
	ungrabAll(): void
	onChangeGrabbed(v: IGrabbable[]): void
}

export default function Grabber() {
	return [...Selector(), GrabberMixin]
}

function GrabberMixin<T extends ISelector>(
	Base: XRControllerConstructor<T>
): ConstructorOf<T & IGrabber> {
	let CoreBase: any = GrabberCore(Base)
	switch (Base.controllerType) {
		case ControllerType.KEYBOARD:
			return GrabberKeyboard(CoreBase) as any

		case ControllerType.QUEST2:
			return GrabberQuest2(CoreBase) as any
	}
	return CoreBase
}

function GrabberCore<T extends ISelector>(Base: ConstructorOf<T>): ConstructorOf<T & IGrabber> {
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class GrabberCoreMixIn extends Base {
		grabbed: IGrabbable[]

		defaults() {
			return {
				...super.defaults(),
				grabbed: [],
			}
		}

		drag(v: Vector2) {
			/* Override with Drag behaviour */
		}

		grab(v: IGrabbable) {
			v.grab()
			if (this.grabbed.indexOf(v) >= 0) return
			this.grabbed.push(v)
			this.onChangeGrabbed(this.grabbed)
		}

		ungrab(v: IGrabbable) {
			v.ungrab()
			if (this.grabbed.indexOf(v) < 0) return
			this.grabbed = this.grabbed.filter(vv => vv !== v)
			this.onChangeGrabbed(this.grabbed)
		}

		ungrabAll() {
			this.grabbed.forEach(v => v.ungrab())
			this.grabbed = []
			this.onChangeGrabbed(this.grabbed)
		}

		onChangeGrabbed(v: IGrabbable[]) {}
	}
	return GrabberCoreMixIn as unknown as ConstructorOf<T & IGrabber>
}
