import ConstructorOf from '@thats/utils/types/ConstructorOf'
import {
	LineBasicMaterial,
	LineSegments,
	Material,
	MeshBasicMaterial,
	MeshLambertMaterial,
	WireframeGeometry,
} from 'three'
import XRMass from '../../core/XRMass'
import { IFocusable } from '../focuser/Focusable'

export interface ISelectable extends XRMass {
	selectMaterial: Material
	isSelectable: boolean
	isSelected: boolean

	select(): void
	deselect(): void
	onChangeSelected(v: boolean): void
}

export default function Selectable<T extends IFocusable>(
	Base: ConstructorOf<T>
): ConstructorOf<T & ISelectable> {
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class SelectableMixIn extends Base {
		private lines: LineSegments
		selectMaterial: MeshBasicMaterial
		isSelectable: boolean
		isSelected: boolean

		defaults() {
			return {
				...super.defaults(),
				isSelectable: true,
				selectMaterial: new MeshLambertMaterial({
					transparent: true,
					opacity: 0.4,
					userData: { importance: 1 },
				}),
			}
		}

		init() {
			super.init()
			this.selectMaterial.color = this.material.color
			const wireframe = new WireframeGeometry(this.geometry)
			this.lines = new LineSegments(wireframe)
			this.lines.material = new LineBasicMaterial({ color: this.material.color })
		}

		select() {
			if (this.isSelected) return
			this.isSelected = true
			this.setMaterial(this.selectMaterial)
			this.add(this.lines)
			this.onChangeSelected(true)
		}
		deselect() {
			if (!this.isSelected) return
			this.isSelected = false
			this.removeMaterial(this.selectMaterial)
			this.remove(this.lines)
			this.onChangeSelected(false)
		}

		onChangeSelected(v: boolean) {}
	}
	return SelectableMixIn as unknown as ConstructorOf<T & ISelectable>
}
