import ConstructorOf from '@thats/utils/types/ConstructorOf'
import { Object3D, Raycaster, Vector2 } from 'three'
import KeyboardController from '../../controllers/KeyboardController'
import { ISelector } from './Selector'

export default function SelectorKeyboard<T extends KeyboardController>(
	Base: ConstructorOf<T & ISelector>
): ConstructorOf<T & ISelector> {
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class SelectorKeyboardMixIn extends Base {
		private _justHit = false
		selectRaycaster: Raycaster
		selectMousePos: Vector2

		defaults() {
			return {
				...super.defaults(),
				selectRaycaster: new Raycaster(),
				selectMousePos: new Vector2(),
			}
		}

		onMouseDown(e: MouseEvent) {
			super['onMouseDown'](e)

			// Raycast to see if we've clicked on something
			const viewer = this.project.viewer
			const plyr = this.project.player
			if (!viewer) return
			const { w: width, h: height } = viewer.dims
			const offset = this.mouseDownPos.offset
			this.selectMousePos.x = (offset.x / width) * 2 - 1
			this.selectMousePos.y = 0 - ((offset.y / height) * 2 - 1)

			this.selectRaycaster.setFromCamera(this.selectMousePos, plyr.camera)

			const targets = this.selectorTargets.map(m => m as Object3D)
			const intersects = this.selectRaycaster.intersectObjects(targets)
			const hitTargetIdx = intersects.length > 0 ? targets.indexOf(intersects[0].object) : -1
			this._justHit = hitTargetIdx >= 0

			// If we have, select it
			if (this._justHit) {
				this.select(this.selectorTargets[hitTargetIdx])
			}
		}

		onMouseUp(e: MouseEvent) {
			super['onMouseUp'](e)
			this.deselectAll()
		}
	}
	return SelectorKeyboardMixIn as any
}
