import ConstructorOf from '@thats/utils/types/ConstructorOf'
import { Vector3 } from 'three'
import { XRUpdateParams } from '../../core/XRClass'
import XRController, { XRControllerConstructor } from '../../core/XRController'
import ControllerType from '../../enums/ControllerType'
import WalkerKeyboard from './WalkerKeyboard'
import WalkerQuest2 from './WalkerQuest2'

export interface IWalker extends XRController {
	impulsion: Vector3
	rotation: Vector3
	walkSpeed: number
	strafeSpeed: number
	lookUpSpeed: number
	yawSpeed: number
	pitchSpeed: number
	rollSpeed: number
}

export default function Walker(fly: boolean = false) {
	return [
		function <T extends XRController>(
			Base: XRControllerConstructor<T>
		): ConstructorOf<T & IWalker> {
			const CoreBase: any = WalkerCore(fly, Base)
			switch (Base.controllerType) {
				case ControllerType.KEYBOARD:
					return WalkerKeyboard(CoreBase) as any

				case ControllerType.QUEST2:
					return WalkerQuest2(CoreBase) as any
			}
			return CoreBase
		},
	]
}

function WalkerCore<T extends XRController>(
	fly: boolean,
	Base: ConstructorOf<T>
): ConstructorOf<T & IWalker> {
	const maxTurn = 45
	// eslint-disable-next-line @typescript-eslint/ban-ts-comment
	// @ts-ignore
	class WalkerCoreMixIn extends Base {
		impulsion: Vector3
		rotation: Vector3
		walkSpeed: number
		strafeSpeed: number
		lookUpSpeed: number
		yawSpeed: number
		pitchSpeed: number
		rollSpeed: number

		defaults() {
			return {
				...super.defaults(),
				impulsion: new Vector3(),
				rotation: new Vector3(),
				walkSpeed: 8, // units per second
				strafeSpeed: 4, // units per second
				lookUpSpeed: Math.PI / 2, // rads per second
				yawSpeed: Math.PI / 2, // rads per second
				pitchSpeed: Math.PI / 2, // rads per second
				rollSpeed: Math.PI / 2, // rads per second
			}
		}

		update(p?: XRUpdateParams) {
			const plyr = this.project.player

			if (p?.delta) {
				if (this.impulsion.z) plyr.walkBy(this.impulsion.z * this.walkSpeed * p.delta)
				if (this.impulsion.x) plyr.strafeBy(this.impulsion.x * this.strafeSpeed * p.delta)
				if (this.rotation.y) plyr.yawBy(this.rotation.y * this.yawSpeed * p.delta)
				if (fly) {
					if (this.rotation.x) plyr.pitchBy(this.rotation.x * this.pitchSpeed * p.delta)
					if (this.rotation.z) plyr.rollBy(this.rotation.z * this.rollSpeed * p.delta)
				} else {
					if (this.rotation.x) plyr.lookUpBy(this.rotation.x * this.lookUpSpeed * p.delta)
				}
			}

			super.update(p)
		}
	}
	return WalkerCoreMixIn as any
}
