"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const XRController_1 = __importDefault(require("../core/XRController"));
const ControllerType_1 = __importDefault(require("../enums/ControllerType"));
const KeyCode_1 = __importDefault(require("../enums/KeyCode"));
const MousePos_1 = require("../enums/MousePos");
class KeyboardController extends XRController_1.default {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { controllerType: ControllerType_1.default.KEYBOARD, shiftDown: false, mouseDown: false, keysDown: [], mouseDownPos: new MousePos_1.MousePos(), mouseUpPos: new MousePos_1.MousePos(), mousePos: new MousePos_1.MousePos(), mouseDiff: new three_1.Vector2() });
    }
    init() {
        super.init();
        this._onKeyDown = this.onKeyDown.bind(this);
        this._onKeyUp = this.onKeyUp.bind(this);
        this._handleMouseDown = this.handleMouseDown.bind(this);
        this._onMouseMove = this.onMouseMove.bind(this);
        this._onMouseUp = this.onMouseUp.bind(this);
        window.addEventListener('mousemove', this._onMouseMove);
        window.addEventListener('mousedown', this._handleMouseDown);
        window.addEventListener('mouseup', this._onMouseUp);
        window.addEventListener('keydown', this._onKeyDown);
        window.addEventListener('keyup', this._onKeyUp);
    }
    update(p) {
        this.mouseDiff.set(0, 0);
        super.update(p);
    }
    isKeyDown(keyCode) {
        return this.keysDown.indexOf(keyCode) >= 0;
    }
    handleMouseDown(e) {
        if (e.target !== this.project.viewer.canvas)
            return;
        this.onMouseDown(e);
    }
    onMouseDown(e) {
        this.mouseDownPos = new MousePos_1.MousePos(e);
        this.mouseDown = true;
        this.mouseDiff.set(0, 0);
        this.mousePos = new MousePos_1.MousePos(e);
    }
    onMouseUp(e) {
        this.mouseUpPos = new MousePos_1.MousePos(e);
        this.mouseDown = false;
    }
    onMouseMove(e) {
        const newPos = new MousePos_1.MousePos(e);
        if (this.mousePos && this.mouseDown)
            this.mouseDiff.add(this.mousePos.client.clone().sub(newPos.client));
        this.mousePos = newPos;
    }
    onKeyDown(e) {
        this.shiftDown = e.shiftKey;
        if (this.isKeyDown(e.keyCode))
            return;
        if (Object.values(KeyCode_1.default).indexOf(e.keyCode) >= 0)
            this.keysDown.push(e.keyCode);
    }
    onKeyUp(e) {
        this.shiftDown = e.shiftKey;
        if (!this.isKeyDown(e.keyCode))
            return;
        if (Object.values(KeyCode_1.default).indexOf(e.keyCode) >= 0)
            this.keysDown = this.keysDown.filter(v => v != e.keyCode);
    }
    destroy() {
        window.removeEventListener('mousemove', this._onMouseMove);
        window.removeEventListener('mousedown', this._handleMouseDown);
        window.removeEventListener('mouseup', this._onMouseUp);
        window.removeEventListener('keydown', this._onKeyDown);
        window.removeEventListener('keyup', this._onKeyUp);
        super.destroy();
    }
}
exports.default = KeyboardController;
KeyboardController.controllerType = ControllerType_1.default.KEYBOARD;
//# sourceMappingURL=KeyboardController.js.map