import { CylinderBufferGeometry } from 'three'
import XRMass from '../core/XRMass'

export default class Cylinder extends XRMass {
	radiusTop: number
	radiusBottom: number
	height: number
	radiusSegments: number
	heightSegments: number
	openEnded: boolean
	thetaStart: number
	thetaLength: number

	defaults() {
		return {
			...super.defaults(),
			radiusTop: 1,
			radiusBottom: 1,
			height: 1,
			radiusSegments: 8,
			heightSegments: 1,
			openEnded: false,
			thetaStart: 0,
			thetaLength: Math.PI * 2,
		}
	}

	initGeometry() {
		this.geometry = new CylinderBufferGeometry(
			this.radiusTop,
			this.radiusBottom,
			this.height,
			this.radiusSegments,
			this.heightSegments,
			this.openEnded,
			this.thetaStart,
			this.thetaLength
		)
	}

	growBy(v: number) {
		this.radiusTop *= v
		this.radiusBottom *= v
		this.height *= v
		this.initGeometry()
	}
}
