import { BackSide, Color, ShaderMaterial } from 'three'
import Sphere from './Sphere'

export class SkyDome extends Sphere {
	topColor: Color
	bottomColor: Color

	defaults() {
		return {
			...super.defaults(),
			radius: 500,
			widthSegments: 32,
			heightSegments: 15,
			topColor: new Color(0x0077ff),
			bottomColor: new Color(0xffffff),
		}
	}

	init() {
		super.init()

		const uniforms = {
			topColor: { value: this.topColor },
			bottomColor: { value: this.bottomColor },
			offset: { value: 0 },
			exponent: { value: 0.6 },
		}

		const vertexShader = `
			varying vec3 vWorldPosition;
			void main() {
				vec4 worldPosition = modelMatrix * vec4( position, 1.0 );
				vWorldPosition = worldPosition.xyz;
				gl_Position = projectionMatrix * modelViewMatrix * vec4( position, 1.0 );
			}
		`

		const fragmentShader = `
			uniform vec3 topColor;
			uniform vec3 bottomColor;
			uniform float offset;
			uniform float exponent;
			varying vec3 vWorldPosition;
			void main() {
				float h = normalize( vWorldPosition + offset ).y;
				gl_FragColor = vec4( mix( bottomColor, topColor, max( pow( max( h , 0.0), exponent ), 0.0 ) ), 1.0 );
			}
		`

		this.material = new ShaderMaterial({
			uniforms,
			vertexShader,
			fragmentShader,
			side: BackSide,
		})
		this.materialStack.push(this.material)
	}
}
