"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const Walker_1 = __importDefault(require("../behaviours/walker/Walker"));
const XRScene_1 = __importDefault(require("../core/XRScene"));
const MotionType_1 = __importDefault(require("../enums/MotionType"));
const DirectionalLight_1 = __importDefault(require("../lights/DirectionalLight"));
const HemisphereLight_1 = __importDefault(require("../lights/HemisphereLight"));
const Box_1 = __importDefault(require("../objects/Box"));
const SkyDome_1 = require("../objects/SkyDome");
class BasicScene extends XRScene_1.default {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { controllerBehaviours: [(0, Walker_1.default)(true)] });
    }
    init() {
        super.init();
        this.gfxScene.background = new three_1.Color().setHSL(0.6, 0, 1);
        this.gfxScene.fog = new three_1.Fog(this.gfxScene.background, 0.1, 100);
        const hemiLight = new HemisphereLight_1.default({
            color: new three_1.Color().setHSL(0.6, 1, 0.6),
            groundColor: new three_1.Color().setHSL(0.095, 1, 0.75),
            intensity: 0.8,
        });
        hemiLight.position.set(0, 50, 0);
        this.addLight(hemiLight);
        const dirLight = new DirectionalLight_1.default({
            color: new three_1.Color().setHSL(0.1, 1, 0.95),
            castShadow: true,
            intensity: 1,
        });
        dirLight.position.set(-1, 1.75, 1);
        dirLight.position.multiplyScalar(30);
        this.addLight(dirLight);
        const ground = new Box_1.default({
            width: 1000,
            depth: 1000,
            height: 0.01,
            mass: 0,
            receiveShadow: true,
            material: new three_1.MeshLambertMaterial({
                color: new three_1.Color().setHSL(0.095, 1, 0.75),
            }),
        });
        ground.position.set(0, -1, 0);
        this.addMass(ground, MotionType_1.default.Kinematic);
        const sky = new SkyDome_1.SkyDome();
        this.addObject(sky);
    }
}
exports.default = BasicScene;
//# sourceMappingURL=BasicScene.js.map