import { Color, Fog, MeshLambertMaterial } from 'three'
import Walker from '../behaviours/walker/Walker'
import XRScene from '../core/XRScene'
import MotionType from '../enums/MotionType'
import DirectionalLight from '../lights/DirectionalLight'
import HemisphereLight from '../lights/HemisphereLight'
import Box from '../objects/Box'
import { SkyDome } from '../objects/SkyDome'

export default class BasicScene extends XRScene {
	floor: Box
	light: DirectionalLight

	defaults() {
		return {
			...super.defaults(),
			controllerBehaviours: [Walker(true)],
		}
	}

	init() {
		super.init()

		this.gfxScene.background = new Color().setHSL(0.6, 0, 1)
		this.gfxScene.fog = new Fog(this.gfxScene.background, 0.1, 100)

		const hemiLight = new HemisphereLight({
			color: new Color().setHSL(0.6, 1, 0.6),
			groundColor: new Color().setHSL(0.095, 1, 0.75),
			intensity: 0.8,
		})
		hemiLight.position.set(0, 50, 0)
		this.addLight(hemiLight)

		const dirLight = new DirectionalLight({
			color: new Color().setHSL(0.1, 1, 0.95),
			castShadow: true,
			intensity: 1,
		})
		dirLight.position.set(-1, 1.75, 1)
		dirLight.position.multiplyScalar(30)
		this.addLight(dirLight)

		// GROUND
		const ground = new Box({
			width: 1000,
			depth: 1000,
			height: 0.01,
			mass: 0,
			receiveShadow: true,
			material: new MeshLambertMaterial({
				color: new Color().setHSL(0.095, 1, 0.75),
			}),
		})
		ground.position.set(0, -1, 0)
		this.addMass(ground, MotionType.Kinematic)

		const sky = new SkyDome()
		this.addObject(sky)
	}
}
