"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Playground = void 0;
const rand_1 = __importDefault(require("@thats/utils/math/rand"));
const three_1 = require("three");
const MRRable_1 = __importDefault(require("../behaviours/mrr/MRRable"));
const MRRor_1 = __importDefault(require("../behaviours/mrr/MRRor"));
const Walker_1 = __importDefault(require("../behaviours/walker/Walker"));
const Box_1 = __importDefault(require("../objects/Box"));
const Sphere_1 = __importDefault(require("../objects/Sphere"));
const ButtonParam_1 = __importDefault(require("../params/ButtonParam"));
const BasicScene_1 = __importDefault(require("./BasicScene"));
class Playground extends BasicScene_1.default {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { key: 'playground', label: 'Playground', playerInitPosition: new three_1.Vector3(0, 0, 10), params: {
                addBox: new ButtonParam_1.default({
                    label: 'Add Box',
                    callback: (scene) => __awaiter(this, void 0, void 0, function* () { return scene.addBox(); }),
                }),
                addBall: new ButtonParam_1.default({
                    label: 'Add Ball',
                    callback: (scene) => __awaiter(this, void 0, void 0, function* () { return scene.addBall(); }),
                }),
                deleteSelected: new ButtonParam_1.default({
                    label: 'Delete',
                    callback: (scene) => __awaiter(this, void 0, void 0, function* () { return scene.deleteSelected(); }),
                }),
            }, controllerBehaviours: [...(0, Walker_1.default)(), ...(0, MRRor_1.default)()] });
    }
    init() {
        super.init();
        this.addBox();
    }
    addBox() {
        return __awaiter(this, void 0, void 0, function* () {
            const box = new ((0, MRRable_1.default)(Box_1.default))({
                width: 1,
                height: 1,
                depth: 1,
                mass: 1,
                castShadow: true,
                material: new three_1.MeshLambertMaterial({
                    color: Math.random() * 0xffffff,
                }),
            });
            box.position.set((0, rand_1.default)(-1, 1), 8, (0, rand_1.default)(-1, 1));
            this.addMass(box);
            box.setAngularVelocity((0, rand_1.default)(-4, 4), (0, rand_1.default)(-4, 4), (0, rand_1.default)(-4, 4));
        });
    }
    addBall() {
        return __awaiter(this, void 0, void 0, function* () {
            const ball = new ((0, MRRable_1.default)(Sphere_1.default))({
                mass: 1,
                radius: 0.4,
                castShadow: true,
                material: new three_1.MeshLambertMaterial({
                    color: Math.random() * 0xffffff,
                }),
            });
            ball.position.set((0, rand_1.default)(-1, 1), 8, (0, rand_1.default)(-1, 1));
            this.addMass(ball);
            ball.setAngularVelocity((0, rand_1.default)(-4, 4), (0, rand_1.default)(-4, 4), (0, rand_1.default)(-4, 4));
        });
    }
    deleteSelected() {
        return __awaiter(this, void 0, void 0, function* () {
            this.all.filter(m => !!m['isSelected']).forEach(this.remove.bind(this));
        });
    }
}
exports.Playground = Playground;
exports.default = Playground;
//# sourceMappingURL=Playground.js.map