import rand from '@thats/utils/math/rand'
import { MeshLambertMaterial, Vector3 } from 'three'
import MRRable from '../behaviours/mrr/MRRable'
import MRRor from '../behaviours/mrr/MRRor'
import Walker from '../behaviours/walker/Walker'
import Box from '../objects/Box'
import Sphere from '../objects/Sphere'
import ButtonParam from '../params/ButtonParam'
import BasicScene from './BasicScene'

export class Playground extends BasicScene {
	defaults() {
		return {
			...super.defaults(),
			key: 'playground',
			label: 'Playground',
			playerInitPosition: new Vector3(0, 0, 10),
			params: {
				addBox: new ButtonParam({
					label: 'Add Box',
					callback: async (scene: Playground) => scene.addBox(),
				}),
				addBall: new ButtonParam({
					label: 'Add Ball',
					callback: async (scene: Playground) => scene.addBall(),
				}),
				deleteSelected: new ButtonParam({
					label: 'Delete',
					callback: async (scene: Playground) => scene.deleteSelected(),
				}),
			},
			controllerBehaviours: [...Walker(), ...MRRor()],
		}
	}

	init() {
		super.init()
		this.addBox()
	}

	async addBox() {
		const box: any = new (MRRable(Box))({
			width: 1,
			height: 1,
			depth: 1,
			mass: 1,
			castShadow: true,
			material: new MeshLambertMaterial({
				color: Math.random() * 0xffffff,
			}),
		})
		box.position.set(rand(-1, 1), 8, rand(-1, 1))
		this.addMass(box)
		box.setAngularVelocity(rand(-4, 4), rand(-4, 4), rand(-4, 4))
	}

	async addBall() {
		const ball = new (MRRable(Sphere))({
			mass: 1,
			radius: 0.4,
			castShadow: true,
			material: new MeshLambertMaterial({
				color: Math.random() * 0xffffff,
			}),
		})
		ball.position.set(rand(-1, 1), 8, rand(-1, 1))
		this.addMass(ball)
		ball.setAngularVelocity(rand(-4, 4), rand(-4, 4), rand(-4, 4))
	}

	async deleteSelected() {
		this.all.filter(m => !!m['isSelected']).forEach(this.remove.bind(this))
	}
}
export default Playground
