import { Matrix4, REVISION, Vector3 } from 'three'

export const iterateGeometries = (function () {
	const inverse = new Matrix4()
	return function (root, options, cb) {
		// MOD (yandeu): Update to three.js r123
		// compatibility fix for three.js >= r123 (Dezember 2020)
		if (parseInt(REVISION) >= 123) inverse.copy(root.matrixWorld).invert()
		else inverse.getInverse(root.matrixWorld)

		const scale = new Vector3()
		scale.setFromMatrixScale(root.matrixWorld)
		root.traverse(mesh => {
			const transform = new Matrix4()
			if (
				mesh.isMesh &&
				// MOD (yandeu): No need to check if name is 'Sky'
				// mesh.name !== 'Sky' &&
				(options.includeInvisible || (mesh.el && mesh.el.object3D.visible) || mesh.visible)
			) {
				if (mesh === root) {
					transform.identity()
				} else {
					mesh.updateWorldMatrix(true)
					transform.multiplyMatrices(inverse, mesh.matrixWorld)
				}
				// todo: might want to return null xform if this is the root so that callers can avoid multiplying
				// things by the identity matrix
				cb(
					mesh.geometry.isBufferGeometry
						? mesh.geometry.attributes.position.array
						: mesh.geometry.vertices,
					transform.elements,
					mesh.geometry.index ? mesh.geometry.index.array : null
				)
			}
		})
	}
})()
export default iterateGeometries
