import ShapeFit from '../../enums/ShapeFit'
import ShapeType from '../../enums/ShapeType'
import computeBounds from '../computeBounds'
import computeHalfExtents from '../computeHalfExtents'
import computeScale from '../computeScale'
import finishCollisionShape from '../finishCollisionShape'
import setDefaultOptions from './setDefaultOptions'

export default function createBoxShape(vertices, matrices, matrixWorld, options: any = {}) {
	options.type = ShapeType.BOX
	setDefaultOptions(options)

	if (options.fit === ShapeFit.ALL) {
		options.halfExtents = computeHalfExtents(
			computeBounds(vertices, matrices),
			options.minHalfExtent,
			options.maxHalfExtent
		)
	}

	const btHalfExtents = new Ammo.btVector3(
		options.halfExtents.x,
		options.halfExtents.y,
		options.halfExtents.z
	)
	const collisionShape = new Ammo.btBoxShape(btHalfExtents)
	Ammo.destroy(btHalfExtents)

	finishCollisionShape(collisionShape, options, computeScale(matrixWorld, options))
	return collisionShape
}
