import ShapeFit from '../../enums/ShapeFit'
import ShapeType from '../../enums/ShapeType'
import computeBounds from '../computeBounds'
import computeHalfExtents from '../computeHalfExtents'
import computeScale from '../computeScale'
import finishCollisionShape from '../finishCollisionShape'
import setDefaultOptions from './setDefaultOptions'

export default function createConeShape(vertices, matrices, matrixWorld, options: any = {}) {
	options.type = ShapeType.CONE
	setDefaultOptions(options)

	if (options.fit === ShapeFit.ALL) {
		options.halfExtents = computeHalfExtents(
			computeBounds(vertices, matrices),
			options.minHalfExtent,
			options.maxHalfExtent
		)
	}

	const { x, y, z } = options.halfExtents
	const collisionShape = (() => {
		switch (options.cylinderAxis) {
			case 'y':
				return new Ammo.btConeShape(Math.max(x, z), y * 2)
			case 'x':
				return new Ammo.btConeShapeX(Math.max(y, z), x * 2)
			case 'z':
				return new Ammo.btConeShapeZ(Math.max(x, y), z * 2)
		}
		return null
	})()

	finishCollisionShape(collisionShape, options, computeScale(matrixWorld, options))
	return collisionShape
}
