"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ShapeFit_1 = __importDefault(require("../../enums/ShapeFit"));
const ShapeType_1 = __importDefault(require("../../enums/ShapeType"));
const computeBounds_1 = __importDefault(require("../computeBounds"));
const computeHalfExtents_1 = __importDefault(require("../computeHalfExtents"));
const computeScale_1 = __importDefault(require("../computeScale"));
const finishCollisionShape_1 = __importDefault(require("../finishCollisionShape"));
const setDefaultOptions_1 = __importDefault(require("./setDefaultOptions"));
function createCylinderShape(vertices, matrices, matrixWorld, options = {}) {
    options.type = ShapeType_1.default.CYLINDER;
    (0, setDefaultOptions_1.default)(options);
    if (options.fit === ShapeFit_1.default.ALL) {
        options.halfExtents = (0, computeHalfExtents_1.default)((0, computeBounds_1.default)(vertices, matrices), options.minHalfExtent, options.maxHalfExtent);
    }
    const btHalfExtents = new Ammo.btVector3(options.halfExtents.x, options.halfExtents.y, options.halfExtents.z);
    const collisionShape = (() => {
        switch (options.cylinderAxis) {
            case 'y':
                return new Ammo.btCylinderShape(btHalfExtents);
            case 'x':
                return new Ammo.btCylinderShapeX(btHalfExtents);
            case 'z':
                return new Ammo.btCylinderShapeZ(btHalfExtents);
        }
        return null;
    })();
    Ammo.destroy(btHalfExtents);
    (0, finishCollisionShape_1.default)(collisionShape, options, (0, computeScale_1.default)(matrixWorld, options));
    return collisionShape;
}
exports.default = createCylinderShape;
//# sourceMappingURL=createCylinderShape.js.map