import ShapeFit from '../../enums/ShapeFit'
import finishCollisionShape from '../finishCollisionShape'
import setDefaultOptions from './setDefaultOptions'

export default function createHeightfieldTerrainShape(options: any = {}) {
	setDefaultOptions(options)

	if (options.fit === ShapeFit.ALL) {
		console.warn('cannot use fit: all with type: heightfield')
		return null
	}
	const heightfieldDistance = options.heightfieldDistance || 1
	const heightfieldData = options.heightfieldData || []
	const heightScale = options.heightScale || 0
	const upAxis = options.hasOwnProperty('upAxis') ? options.upAxis : 1 // x = 0; y = 1; z = 2
	const hdt = (() => {
		switch (options.heightDataType) {
			case 'short':
				return Ammo['PHY_SHORT']
			case 'float':
				return Ammo['PHY_FLOAT']
			default:
				return Ammo['PHY_FLOAT']
		}
	})()
	const flipQuadEdges = options.hasOwnProperty('flipQuadEdges') ? options.flipQuadEdges : true

	const heightStickLength = heightfieldData.length
	const heightStickWidth = heightStickLength > 0 ? heightfieldData[0].length : 0

	const data = Ammo._malloc(heightStickLength * heightStickWidth * 4)
	const ptr = data / 4

	let minHeight = Number.POSITIVE_INFINITY
	let maxHeight = Number.NEGATIVE_INFINITY
	let index = 0
	for (let l = 0; l < heightStickLength; l++) {
		for (let w = 0; w < heightStickWidth; w++) {
			const height = heightfieldData[l][w]
			Ammo.HEAPF32[ptr + index] = height
			index++
			minHeight = Math.min(minHeight, height)
			maxHeight = Math.max(maxHeight, height)
		}
	}

	const collisionShape = new Ammo.btHeightfieldTerrainShape(
		heightStickWidth,
		heightStickLength,
		data,
		heightScale,
		minHeight,
		maxHeight,
		upAxis,
		hdt,
		flipQuadEdges
	)

	const scale = new Ammo.btVector3(heightfieldDistance, 1, heightfieldDistance)
	collisionShape.setLocalScaling(scale)
	Ammo.destroy(scale)

	collisionShape['heightfieldData'] = data

	finishCollisionShape(collisionShape, options)
	return collisionShape
}
