import Col from '@thatweb/core-mui/components/Col'
import Row from '@thatweb/core-mui/components/Row'
import Typo from '@thatweb/core-mui/components/typo/Typo'
import React, { useState } from 'react'
import loadAmmo from '../utils/loadAmmo'
import XRProjectControls from './XRProjectControls'
import XRProjectViewerState from './XRProjectViewerState'
import XRSceneViewer from './XRSceneViewer'

interface XRSceneViewerProps {
	sceneIdx: number
}

export function XRProjectViewer(props: XRSceneViewerProps) {
	const { sceneIdx } = props
	const [ammoLoaded, setAmmoLoaded] = useState<boolean>(false)

	React.useEffect(() => {
		if (ammoLoaded) return
		loadAmmo('/assets', () => setAmmoLoaded(true))
	}, [ammoLoaded])

	if (!ammoLoaded) return <Typo variant="h2">Loading</Typo>

	return (
		<XRProjectViewerState sceneIdx={sceneIdx}>
			<Col padding={2} sx={{ flex: 1 }}>
				<Row spacing={2} direction="row" sx={{ flex: 1, overflow: 'hidden' }}>
					<XRSceneViewer />
					<XRProjectControls />
				</Row>
			</Col>
		</XRProjectViewerState>
	)
}
export default XRProjectViewer
